import React, { Component } from 'react';
import classnames from 'classnames';
import PropTypes from 'prop-types';
import map from 'lodash/map';
import filter from 'lodash/filter';
import isEmpty from 'lodash/isEmpty';
import isNull from 'lodash/isNull';
import noop from 'lodash/noop';
import uuid from 'uuid';
import { compareLabelAttribute } from './../../_common/utils';
import Icon from './../icon/icon';
import './suggestion.less';

export default class Suggestion extends Component {

    constructor(props) {
        super(props);
        this.overlayTopPosition = 0;
        this.overlayTopPositionOffset = 5;
        this.overlayWidth = 0;
        this.state = {
            suggestions: this.props.suggestions,
            suggestionsCopy: this.props.suggestions,
            showSelectBox: false,
            value: '',
            selectedItem: {}
        };
        this.handleOnInputChange = this.handleOnInputChange.bind(this);
        this.handleKeydown = this.handleKeydown.bind(this);
        this.handleSelect = this.handleSelect.bind(this);
        this.handleClick = this.handleClick.bind(this);
    }

    componentDidMount() {
        document.addEventListener('keydown', this.handleKeydown);
        document.addEventListener('click', this.handleClick);
    }

    componentWillUnmount() {
        document.removeEventListener('keydown', this.handleKeydown);
        document.removeEventListener('click', this.handleClick);
    }

    handleClick(event) {
        /* istanbul ignore if */
        if (document.getElementById(event.target.getAttribute('id')) !== null) {
            const {
                value
            } = this.state;
            const element = document.getElementById(event.target.getAttribute('id'));
            const positionTop = document.getElementById(event.target.getAttribute('id')).parentNode.offsetTop
                + this.overlayTopPositionOffset
                + document.getElementById(event.target.getAttribute('id')).offsetHeight * 1.5;

            if (!isEmpty(value)) {
                this.handleSelect(event);
            }
            this.setState({
                showSelectBox: false
            });

            if (!isNull(element)) {
                const inputHeight = element.offsetHeight;

                this.overlayTopPosition = positionTop + inputHeight;
                this.overlayWidth = element.offsetWidth - 12;
            }

        }
    }

    handleKeydown(event) {
        const {
            suggestionsCopy
        } = this.state;

        switch (event.which) {
            case 8: // Backspace
                this.setState({
                    suggestions: suggestionsCopy
                });
                break;
            case 27: // ESC
                this.setState({
                    showSelectBox: false,
                    value: ''
                });
                break;
            default:
                break;
        }
    }

    handleOnInputChange(event) {
        const inputValue = event.target.value;
        const {
            suggestions
        } = this.state;
        const {
            onChange
        } = this.props;
        const filteredItems = filter(suggestions, (item) => {
            if (isEmpty(inputValue)) {
                this.setState({
                    showSelectBox: false
                });
            } else {
                this.setState({
                    showSelectBox: inputValue.length > 1
                });
            }

            return item.label.includes(inputValue);
        });

        if (isEmpty(filteredItems)) {
            this.setState({
                showSelectBox: false
            });
        } else {
            this.setState({
                suggestions: filteredItems.sort(compareLabelAttribute)
            });
        }
        this.setState({
            value: event.target.value
        });
        onChange({
            value: inputValue
        });
    }

    /* istanbul ignore next */
    handleSelect(event) {
        const {
            onSelect
        } = this.props;
        const selectedItem = event.target;

        this.setState({
            showSelectBox: false,
            value: event.target.textContent
        });
        onSelect({
            id: selectedItem.getAttribute('data-attribute-id'),
            label: selectedItem.getAttribute('data-attribute-label'),
            value: selectedItem.getAttribute('data-attribute-value')
        });
    }

    render() {
        const {
            id,
            subClass,
            disabled,
            placeholder,
            showIcon,
            iconType,
            width
        } = this.props;
        const {
            suggestions,
            showSelectBox,
            value
        } = this.state;
        const className = classnames(
            'tsdev-suggestion', subClass
        );
        const inputClassName = classnames('tsdev-suggestion-input', disabled ? 'disabled' : '');
        const inlineStyle = {
            width: width
        };
        const inlineStyleSelectBox = {
            display: showSelectBox ? 'block' : 'none',
            top: this.overlayTopPosition,
            width: this.overlayWidth
        };
        const inlineStyleIcon = {
            display: showIcon ? 'block' : 'none'
        };
        const suggestionList = map(suggestions, (suggestion, index) => {
            return (
                <div key={`suggestion-${index}`}>
                    <div className="tsdev-suggestion-suggestion-box-item"
                        data-attribute-id={suggestion.id}
                        data-attribute-label={suggestion.label}
                        data-attribute-value={suggestion.value}
                        onClick={this.handleClick}>
                        {suggestion.label}
                    </div>
                </div>
            );
        });

        return (
            <div id={id}
                className={className}
                style={inlineStyle}>
                <div className="input-wrapper">
                    <input type="text"
                        id={`${id}-input`}
                        value={value}
                        onChange={this.handleOnInputChange}
                        onKeyPress={this.handleKeydown}
                        disabled={disabled}
                        placeholder={placeholder}
                        className={inputClassName} />
                    <div className="input-wrapper-icon"
                        style={inlineStyleIcon}>
                        <Icon iconType={iconType} />
                    </div>
                </div>
                <div className="tsdev-suggestion-suggestion-box"
                    style={inlineStyleSelectBox}>
                    {suggestionList}
                </div>
            </div>
        );
    }
}

Suggestion.propTypes = {
    disabled: PropTypes.bool,
    iconType: PropTypes.string,
    id: PropTypes.string,
    onSelect: PropTypes.func,
    onChange: PropTypes.func,
    placeholder: PropTypes.string,
    showIcon: PropTypes.bool,
    subClass: PropTypes.string,
    suggestions: PropTypes.arrayOf(PropTypes.shape({
        id: PropTypes.string,
        label: PropTypes.string,
        value: PropTypes.oneOfType([
            PropTypes.string,
            PropTypes.number
        ])
    })),
    width: PropTypes.oneOfType([
        PropTypes.string,
        PropTypes.number
    ])
};

Suggestion.defaultProps = {
    disabled: false,
    iconType: 'search',
    id: uuid.v4(),
    onSelect: noop,
    onChange: noop,
    placeholder: 'Placeholder',
    showIcon: true,
    subClass: '',
    suggestions: [
        {
            id: '1',
            label: '1 - Suggestion',
            value: 1
        },
        {
            id: '2',
            label: '11 - Suggestion',
            value: 11
        },
        {
            id: '3',
            label: '112 - Suggestion',
            value: 112
        },
        {
            id: '4',
            label: '2 - Suggestion',
            value: 2
        },
        {
            id: '5',
            label: '21 - Suggestion',
            value: 21
        },
        {
            id: '6',
            label: '212 - Suggestion',
            value: 212
        },
        {
            id: '7',
            label: '22 - Suggestion',
            value: 22
        },
        {
            id: '8',
            label: '222 - Suggestion',
            value: 222
        }
    ],
    width: 500
};

Suggestion.displayName = 'Suggestion';
